/**************************************************************************
**
** This file is part of Qt Creator
**
** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
**
** Contact: Nokia Corporation (qt-info@nokia.com)
**
**
** GNU Lesser General Public License Usage
**
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this file.
** Please review the following information to ensure the GNU Lesser General
** Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** Other Usage
**
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
**************************************************************************/

#ifndef RUNCONFIGURATIONMODEL_H
#define RUNCONFIGURATIONMODEL_H

#include <QAbstractItemModel>

namespace ProjectExplorer {
class Target;
class RunConfiguration;

// Documentation inside.
class RunConfigurationModel : public QAbstractListModel
{
    Q_OBJECT
public:
    explicit RunConfigurationModel(Target *target, QObject *parent = 0);

    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    RunConfiguration *runConfigurationAt(int i);
    RunConfiguration *runConfigurationFor(const QModelIndex &idx);
    QModelIndex indexFor(RunConfiguration *rc);
private slots:
    void addedRunConfiguration(ProjectExplorer::RunConfiguration*);
    void removedRunConfiguration(ProjectExplorer::RunConfiguration*);
    void displayNameChanged();
private:
    Target *m_target;
    QList<RunConfiguration *> m_runConfigurations;
};

}

#endif // RUNCONFIGURATIONMODEL_H
